@@ -1,4 +1,5 @@
 import pandas as pd
+import numpy as np
 import glob
 import os
 from statistics import mean
@@ -91,6 +92,7 @@ class GbmCellData():
         groups,
         file_location="Data_Raw/*/*_org?_Shortest_Distance_to_Surfaces_Surfaces*.csv",
         invasion_threshold_details=[0.75, 'DMSO'],
+        normalization=False,
         linreg_analysis=False,
         export_data=False,
         export_data_location="Data_Processed/",
@@ -255,22 +257,25 @@ class GbmCellData():
                 inv_raw_file, sheet_name='inv_raw_bygroup', index=False)
             self.inv_raw_bygroup_tidy.to_excel(
                 inv_raw_file, sheet_name='inv_raw_bygroup_tidy', index=False)
+            pd.DataFrame([self.inv_thresh], columns=['Threshold']).to_excel(
+                inv_raw_file, sheet_name='inv_threshold', index=False)
             inv_raw_file.save()
 
             # Fraction greater than threshold data
-            inv_norm_file = pd.ExcelWriter(
-                export_data_location + 'inv_norm.xlsx')
-            self.inv_norm.to_excel(
-                inv_norm_file, sheet_name='inv_norm', index=False)
-            self.inv_norm_tidy.to_excel(
-                inv_norm_file, sheet_name='inv_norm_tidy', index=False)
-            self.inv_norm_bygroup.to_excel(
-                inv_norm_file, sheet_name='inv_norm_bygroup', index=False)
-            self.inv_norm_bygroup_tidy.to_excel(
-                inv_norm_file, sheet_name='inv_norm_bygroup_tidy', index=False)
-            inv_norm_file.save()
+            if normalization is True:
+                inv_norm_file = pd.ExcelWriter(
+                    export_data_location + 'inv_norm.xlsx')
+                self.inv_norm.to_excel(
+                    inv_norm_file, sheet_name='inv_norm', index=False)
+                self.inv_norm_tidy.to_excel(
+                    inv_norm_file, sheet_name='inv_norm_tidy', index=False)
+                self.inv_norm_bygroup.to_excel(
+                    inv_norm_file, sheet_name='inv_norm_bygroup', index=False)
+                self.inv_norm_bygroup_tidy.to_excel(
+                    inv_norm_file, sheet_name='inv_norm_bygroup_tidy', index=False)
+                inv_norm_file.save()
 
-            print("GBM cell data exported to excel files.")
+            print("Exported GBM cell data to excel files.")
 
         # ===== STATISTICS =====
 
@@ -310,10 +315,11 @@ class GbmCellData():
             stats.save_to_txt(inv_raw_anova_filename, inv_raw_anova_text)
 
             # Fraction greater than threshold anova
-            inv_norm_anova_text = stats.anova_results_as_text(
-                self.inv_norm_bygroup)
-            inv_norm_anova_filename = export_stats_location + 'inv_norm_anova.txt'
-            stats.save_to_txt(inv_norm_anova_filename, inv_norm_anova_text)
+            if normalization is True:
+                inv_norm_anova_text = stats.anova_results_as_text(
+                    self.inv_norm_bygroup)
+                inv_norm_anova_filename = export_stats_location + 'inv_norm_anova.txt'
+                stats.save_to_txt(inv_norm_anova_filename, inv_norm_anova_text)
 
             # Linear regression
             if linreg_analysis is True:
@@ -337,10 +343,12 @@ class GbmCellData():
                 figs.bargraph(self.cn_bygroup, group_colors=colors)
                 figs.bargraph(
                     self.inv_raw_bygroup, group_colors=colors, y_label='Number of Invaded Cells')
-                figs.bargraph(
-                    self.inv_norm_bygroup, group_colors=colors, y_label='Invaded Cells \n (as fraction of total)')
                 figs.boxplot(self.dts_tidy, group_colors=colors)
-
+                figs.norm_stacked_kde(
+                    self.dts_bygroup_tidy, group_colors=colors)
+                if normalization is True:
+                    figs.bargraph(
+                        self.inv_norm_bygroup, group_colors=colors, y_label='Invaded Cells \n (as fraction of total)')
                 if linreg_analysis is True:
                     figs.linregplot(self.inv_tidy, self.cn_tidy,
                                     group_colors=colors)
@@ -362,17 +370,11 @@ class GbmCellData():
                 self.inv_raw_bargraph = figs.bargraph(
                     self.inv_raw_bygroup, group_colors=colors, y_label='Number of Invaded Cells')
                 inv_raw_bargraph_filename = export_figs_location + 'inv_raw_bargraph.pdf'
-                figs.figtofile(self.inv_raw_bargraph, inv_raw_bargraph_filename)
+                figs.figtofile(self.inv_raw_bargraph,
+                               inv_raw_bargraph_filename)
                 print("Saved bargraph of number of invading cells as PDF.")
                 plt.close()
 
-                self.inv_norm_bargraph = figs.bargraph(
-                    self.inv_norm_bygroup, group_colors=colors, y_label='Invaded Cells \n (as fraction of total)')
-                inv_norm_bargraph_filename = export_figs_location + 'inv_norm_bargraph.pdf'
-                figs.figtofile(self.inv_norm_bargraph, inv_norm_bargraph_filename)
-                print("Saved bargraph of fraction of invading cells as PDF.")
-                plt.close()
-
                 self.dts_boxplot = figs.boxplot(
                     self.dts_tidy, group_colors=colors)
                 dts_boxplot_filename = export_figs_location + 'dts_boxplot.pdf'
@@ -380,6 +382,23 @@ class GbmCellData():
                 print("Saved boxplot of distace to surface values as PDF.")
                 plt.close()
 
+                self.kde = figs.norm_stacked_kde(
+                    self.dts_bygroup_tidy, group_colors=colors)
+                kde_filename = export_figs_location + 'kde.pdf'
+                figs.figtofile(self.kde, kde_filename)
+                print(
+                    'Saved normalized, stacked KDE plot of distance surface values as PDF.')
+                plt.close()
+
+                if normalization is True:
+                    self.inv_norm_bargraph = figs.bargraph(
+                        self.inv_norm_bygroup, group_colors=colors, y_label='Invaded Cells \n (as fraction of total)')
+                    inv_norm_bargraph_filename = export_figs_location + 'inv_norm_bargraph.pdf'
+                    figs.figtofile(self.inv_norm_bargraph,
+                                   inv_norm_bargraph_filename)
+                    print("Saved bargraph of fraction of invading cells as PDF.")
+                    plt.close()
+
                 if linreg_analysis is True:
                     self.linregplot = figs.linregplot(
                         self.inv_tidy, self.cn_tidy, group_colors=colors)
@@ -563,6 +582,17 @@ class OrganoidData():
         self.vol = self.fullvol - self.extract_org_data(self.vol_files_dict)
         self.sa = self.extract_org_data(self.sa_files_dict) - self.fullarea
 
+        # Replacing negative vol or sa values with NaN
+        for data in [self.vol, self.sa]:
+            for values in data.values.tolist():
+                if any(value < 0 for value in values):
+                    print(
+                        "Note: Negative value(s) found and removed in processed surface area or volume data.")
+            for col, value in data.items():
+                value = float(value)
+                if value < 0:
+                    data.replace(value, np.NaN, inplace=True)
+
         # Making bygroup and tidyform dfs
         self.vol_bygroup, self.vol_tidy, self.vol_bygroup_tidy = \
             mk_comb_and_tidy_dfs(self.vol, 'Volume')
